@@ -36,7 +36,7 @@ constexpr unsigned API_VERSION_2_2 = ZIMG_MAKE_API_VERSION(2, 2);
 thread_local zimg_error_code_e g_last_error = ZIMG_ERROR_SUCCESS;
 thread_local std::string g_last_error_msg;
 
-constexpr unsigned VERSION_INFO[] = { 2, 7, 5 };
+constexpr unsigned VERSION_INFO[] = { 2, 8, 0 };
 
 
 template <class T, class U>
@@ -232,15 +232,15 @@ zimg::colorspace::MatrixCoefficients translate_matrix(zimg_matrix_coefficients_e
 
 	static SM_CONSTEXPR_14 const zimg::static_map<zimg_matrix_coefficients_e, zimg::colorspace::MatrixCoefficients, 13> map{
 		{ ZIMG_MATRIX_RGB,                      MatrixCoefficients::RGB },
-		{ ZIMG_MATRIX_709,                      MatrixCoefficients::REC_709 },
+		{ ZIMG_MATRIX_BT709,                    MatrixCoefficients::REC_709 },
 		{ ZIMG_MATRIX_UNSPECIFIED,              MatrixCoefficients::UNSPECIFIED },
 		{ ZIMG_MATRIX_FCC,                      MatrixCoefficients::FCC },
-		{ ZIMG_MATRIX_470BG,                    MatrixCoefficients::REC_601 },
-		{ ZIMG_MATRIX_170M,                     MatrixCoefficients::REC_601 },
-		{ ZIMG_MATRIX_240M,                     MatrixCoefficients::SMPTE_240M },
+		{ ZIMG_MATRIX_BT470_BG,                 MatrixCoefficients::REC_601 },
+		{ ZIMG_MATRIX_ST170_M,                  MatrixCoefficients::REC_601 },
+		{ ZIMG_MATRIX_ST240_M,                  MatrixCoefficients::SMPTE_240M },
 		{ ZIMG_MATRIX_YCGCO,                    MatrixCoefficients::YCGCO },
-		{ ZIMG_MATRIX_2020_NCL,                 MatrixCoefficients::REC_2020_NCL },
-		{ ZIMG_MATRIX_2020_CL,                  MatrixCoefficients::REC_2020_CL },
+		{ ZIMG_MATRIX_BT2020_NCL,               MatrixCoefficients::REC_2020_NCL },
+		{ ZIMG_MATRIX_BT2020_CL,                MatrixCoefficients::REC_2020_CL },
 		{ ZIMG_MATRIX_CHROMATICITY_DERIVED_NCL, MatrixCoefficients::CHROMATICITY_DERIVED_NCL },
 		{ ZIMG_MATRIX_CHROMATICITY_DERIVED_CL,  MatrixCoefficients::CHROMATICITY_DERIVED_CL },
 		{ ZIMG_MATRIX_ICTCP,                    MatrixCoefficients::REC_2100_ICTCP },
@@ -253,16 +253,16 @@ zimg::colorspace::TransferCharacteristics translate_transfer(zimg_transfer_chara
 	using zimg::colorspace::TransferCharacteristics;
 
 	static SM_CONSTEXPR_14 const zimg::static_map<zimg_transfer_characteristics_e, TransferCharacteristics, 15> map{
-		{ ZIMG_TRANSFER_709,           TransferCharacteristics::REC_709 },
+		{ ZIMG_TRANSFER_BT709,         TransferCharacteristics::REC_709 },
 		{ ZIMG_TRANSFER_UNSPECIFIED,   TransferCharacteristics::UNSPECIFIED },
-		{ ZIMG_TRANSFER_240M,          TransferCharacteristics::SMPTE_240M },
-		{ ZIMG_TRANSFER_601,           TransferCharacteristics::REC_709 },
-		{ ZIMG_TRANSFER_470_M,         TransferCharacteristics::REC_470_M },
-		{ ZIMG_TRANSFER_470_BG,        TransferCharacteristics::REC_470_BG },
+		{ ZIMG_TRANSFER_ST240_M,       TransferCharacteristics::SMPTE_240M },
+		{ ZIMG_TRANSFER_BT601,         TransferCharacteristics::REC_709 },
+		{ ZIMG_TRANSFER_BT470_M,       TransferCharacteristics::REC_470_M },
+		{ ZIMG_TRANSFER_BT470_BG,      TransferCharacteristics::REC_470_BG },
 		{ ZIMG_TRANSFER_IEC_61966_2_4, TransferCharacteristics::XVYCC },
 		{ ZIMG_TRANSFER_IEC_61966_2_1, TransferCharacteristics::SRGB },
-		{ ZIMG_TRANSFER_2020_10,       TransferCharacteristics::REC_709 },
-		{ ZIMG_TRANSFER_2020_12,       TransferCharacteristics::REC_709 },
+		{ ZIMG_TRANSFER_BT2020_10,     TransferCharacteristics::REC_709 },
+		{ ZIMG_TRANSFER_BT2020_12,     TransferCharacteristics::REC_709 },
 		{ ZIMG_TRANSFER_LINEAR,        TransferCharacteristics::LINEAR },
 		{ ZIMG_TRANSFER_LOG_100,       TransferCharacteristics::LOG_100 },
 		{ ZIMG_TRANSFER_LOG_316,       TransferCharacteristics::LOG_316 },
@@ -277,14 +277,14 @@ zimg::colorspace::ColorPrimaries translate_primaries(zimg_color_primaries_e prim
 	using zimg::colorspace::ColorPrimaries;
 
 	static SM_CONSTEXPR_14 const zimg::static_map<zimg_color_primaries_e, ColorPrimaries, 12> map{
-		{ ZIMG_PRIMARIES_470_M,       ColorPrimaries::REC_470_M },
-		{ ZIMG_PRIMARIES_470_BG,      ColorPrimaries::REC_470_BG },
-		{ ZIMG_PRIMARIES_709,         ColorPrimaries::REC_709 },
+		{ ZIMG_PRIMARIES_BT470_M,     ColorPrimaries::REC_470_M },
+		{ ZIMG_PRIMARIES_BT470_BG,    ColorPrimaries::REC_470_BG },
+		{ ZIMG_PRIMARIES_BT709,       ColorPrimaries::REC_709 },
 		{ ZIMG_PRIMARIES_UNSPECIFIED, ColorPrimaries::UNSPECIFIED },
-		{ ZIMG_PRIMARIES_170M,        ColorPrimaries::SMPTE_C },
-		{ ZIMG_PRIMARIES_240M,        ColorPrimaries::SMPTE_C },
+		{ ZIMG_PRIMARIES_ST170_M,     ColorPrimaries::SMPTE_C },
+		{ ZIMG_PRIMARIES_ST240_M,     ColorPrimaries::SMPTE_C },
 		{ ZIMG_PRIMARIES_FILM,        ColorPrimaries::FILM },
-		{ ZIMG_PRIMARIES_2020,        ColorPrimaries::REC_2020 },
+		{ ZIMG_PRIMARIES_BT2020,      ColorPrimaries::REC_2020 },
 		{ ZIMG_PRIMARIES_ST428,       ColorPrimaries::XYZ },
 		{ ZIMG_PRIMARIES_ST431_2,     ColorPrimaries::DCI_P3 },
 		{ ZIMG_PRIMARIES_ST432_1,     ColorPrimaries::DCI_P3_D65 },
